@@ -3,10 +3,11 @@ import logging
 import re
 from enum import Enum
 
-from db_hammer.entity_util import update_sql, insert_sql, delete_sql
-
 from db_hammer.csv import start as csv_start
-from db_hammer.sql_exception import SqlException
+from db_hammer.entity_util import update_sql, insert_sql, delete_sql, entity_list, where_entity, where_like_entity, \
+    order_by_pagination, get_entity_primary_key
+from db_hammer.page import PageOutput, PageInput
+from db_hammer.sql_exception import SqlException, FetchRowsException, ExistException
 from db_hammer.util.date import date_to_str
 
 
@@ -32,9 +33,9 @@ class BaseConnection(object):
     def __exit__(self, exc_type, exc_val, exc_tb):
         self.close()
 
-    def select_page_size(self, sql: str, params: dict = None, page_size=50) -> int:
+    def select_page_size(self, sql: str, params: dict = None, page_size=50) -> (int, int):
         """
-        根据sql获取页数
+        根据sql获取 (页数,行数)
         :param params:
         :param sql:
         :param page_size: 分页大小
@@ -43,14 +44,16 @@ class BaseConnection(object):
 
         count_sql = "select COUNT(0) from ( %s ) temp_count" % sql
         count_sql = self.sql_params(count_sql, params)
-        self.log.debug("execute sql :" + count_sql.replace("\n", " "))
+        self.log.debug("执行SQL:" + count_sql.replace("\n", " "))
+        self.log.debug("参数:" + str(params))
         self.cursor.execute(count_sql, params)
         data = self.cursor.fetchone()
-        self.log.debug("fetch rows:" + str(len(data)))
-        num = data[0] // page_size
-        if data[0] > page_size * num:
+        count_rows = data[0]
+        self.log.debug("影响行数:" + str(len(data)))
+        num = count_rows // page_size
+        if count_rows > page_size * num:
             num = num + 1
-        return num
+        return num, count_rows
 
     def sql_params(self, sql: str, params: dict):
         if params is None:
@@ -87,16 +90,14 @@ class BaseConnection(object):
             add_headers = False
 
         start = page_size * (page_start - 1)
-        end = page_size * (page_end - 1)
-        page_sql = """select * FROM (select temp_page.*, rownum row_id from 
-                         ( %s ) temp_page 
-                       where limit %d,%d
-                       """ % (sql, end, start)
+        end = page_size * (page_end - page_start)
+        page_sql = """SELECT * FROM ( %s ) temp_page LIMIT %d,%d """ % (sql, start, end)
+        self.log.debug("执行SQL:" + page_sql.replace("\n", " "))
         page_sql = self.sql_params(page_sql, params)
-        self.log.debug("execute sql :" + page_sql.replace("\n", " "))
+        self.log.debug("参数:" + str(params))
         self.cursor.execute(page_sql, params)
         data = self.cursor.fetchall()
-        self.log.debug("fetch rows:" + str(len(data)))
+        self.log.debug("影响行数:" + str(len(data)))
         if add_headers:
             col_names = self.cursor.description
             list(data).insert(col_names, 0)
@@ -127,7 +128,7 @@ class BaseConnection(object):
         :param sql:
         :return:
         """
-        self.log.debug("execute sql :" + sql.replace("\n", " "))
+        self.log.debug("执行SQL:" + sql.replace("\n", " "))
         _sql = self.sql_params(sql, params)
         self.cursor.execute(_sql, params)
         data = self.cursor.fetchone()
@@ -141,11 +142,12 @@ class BaseConnection(object):
         :param sql:
         :return:
         """
-        self.log.debug("execute sql :" + sql.replace("\n", " "))
+        self.log.debug("执行SQL:" + sql.replace("\n", " "))
+        self.log.debug("参数:" + str(params))
         _sql = self.sql_params(sql, params)
         self.cursor.execute(_sql, params)
         data = self.cursor.fetchall()
-        self.log.debug("fetch rows:" + str(len(data)))
+        self.log.debug("影响行数:" + str(len(data)))
         return data
 
     def select_dict_list(self, sql: str, params=None) -> list:
@@ -155,11 +157,12 @@ class BaseConnection(object):
         :param sql:
         :return:
         """
-        self.log.debug("execute sql :" + sql.replace("\n", " "))
+        self.log.debug("执行SQL:" + sql.replace("\n", " "))
+        self.log.debug("参数:" + str(params))
         _sql = self.sql_params(sql, params)
         self.cursor.execute(_sql, params)
         data = self.cursor.fetchall()
-        self.log.debug("fetch rows:" + str(len(data)))
+        self.log.debug("影响行数:" + str(len(data)))
         col_names = self.cursor.description
         return self._data_to_map(col_names, data)
 
@@ -192,28 +195,30 @@ class BaseConnection(object):
         :param sql:
         :return:
         """
-        # self.log.debug("execute sql :" + sql.replace("\n", " "))
-        self.cursor.execute(sql, params)
+        _sql = self.sql_params(sql, params)
+        self.log.debug("执行SQL:" + sql.replace("\n", " "))
+        self.log.debug("参数:" + str(params))
+        self.cursor.execute(_sql, params)
         i = self.cursor.rowcount
-        self.log.debug("fetch rows:" + str(i))
+        self.log.debug("影响行数:" + str(i))
         return i
 
     def close(self):
         self.conn.close()
-        self.log.debug("conn close")
+        # self.log.debug("关闭连接")
 
     def rollback(self):
         """回滚事务"""
-        self.log.debug("rollback start")
+        # self.log.debug("回滚事务开始")
         self.conn.rollback()
-        self.log.debug("rollback end")
+        self.log.debug("回滚事务完成")
 
     def commit(self):
         """提交事务，如果有对数据库进更新，需要手动提交事务
         """
-        self.log.debug("commit start")
+        # self.log.debug("提交事务开始")
         self.conn.commit()
-        self.log.debug("commit end")
+        self.log.debug("提交事务完成")
         return
 
     def db_data_type_mapping(self):
@@ -382,36 +387,116 @@ class BaseConnection(object):
                   callback=outingCallback,
                   log=self.log)
 
-    def insert_entity(self, entity):
+    def insert_entity(self, entity, commit=True):
         """传入实体保存到数据库"""
         sql, values = insert_sql(entity=entity)
-        self.log.debug(sql)
-        self.log.debug(f"params:{values}")
         sql = self.sql_params(sql, values)
         i = self.execute(sql, values)
-        self.commit()
-        return i
+        primary_key = self.cursor.lastrowid
+        if primary_key != 0:
+            setattr(entity, get_entity_primary_key(entity)[0], primary_key)
+        if i != 1:
+            raise FetchRowsException(f"影响条数为{i}")
+        if commit:
+            self.commit()
+        return entity
 
-    def update_entity(self, entity, pass_null=False):
+    def update_entity(self, entity, pass_null=False, commit=True, check_exist=False) -> None:
+        if check_exist:
+            es = self.select_entity_by_pk(entity=entity)
+            if es is None:
+                raise ExistException(f"找不到要更新的实体")
         sql, values = update_sql(entity=entity, pass_null=pass_null)
         self.log.debug(f"SQL:{sql}")
         self.log.debug(f"params:{values}")
         sql = self.sql_params(sql, values)
         i = self.execute(sql, values)
-        self.commit()
-        return i
+        if commit:
+            self.commit()
 
-    def delete_entity(self, entity):
+    def delete_entity(self, entity, commit=True) -> int:
         sql, values = delete_sql(entity=entity)
         self.log.debug(sql)
         self.log.debug(f"params:{values}")
         sql = self.sql_params(sql, values)
         i = self.execute(sql, values)
-        self.commit()
+        if commit:
+            self.commit()
         return i
 
-    def select_entity_list(self):
-        print("select_list_by_columns")
+    def select_entity_list(self, cls=None, sql=None, params=None, like_entity=None, entity=None, rel="AND",
+                           page: PageInput = None):
+        if sql is None:
+            if like_entity is not None:
+                if cls is None:
+                    cls = like_entity.__class__
+                table_name = getattr(like_entity, "__table_name__")
+            elif entity is not None:
+                if cls is None:
+                    cls = entity.__class__
+                table_name = getattr(entity, "__table_name__")
+            else:
+                table_name = getattr(cls, "__table_name__")
 
-    def select_entity_one(self):
-        print("select_one_by_columns")
+            sql = "SELECT * FROM " + table_name
+        if params is None:
+            if like_entity is not None:
+                where, values = where_like_entity(entity=like_entity, rel=rel)
+                sql += where
+                params = values
+            elif entity is not None:
+                where, values = where_entity(entity=entity, rel=rel)
+                sql += where
+                params = values
+        if sql is None:
+            raise Exception("查询SQL不能为空")
+        if page is not None:
+            pages, rowsNumber = self.select_page_size(sql=sql, params=params,
+                                                      page_size=page.page_size)
+            order_by = order_by_pagination(entity=cls, pagination=page)
+            dict_list = self.select_dict_page_list(sql=f"{sql} {order_by}", params=params,
+                                                   page_size=page.page_size,
+                                                   page_start=page.page_start)
+            values = entity_list(dict_list=dict_list, cls=cls)
+            pageOut = PageOutput()
+            pageOut.rows = values
+            pageOut.page_start = page.page_start
+            pageOut.page_size = page.page_size
+            pageOut.page_number = pages
+            pageOut.rows_number = rowsNumber
+            pageOut.sort_by = page.sort_by
+            pageOut.descending = page.descending
+            return pageOut
+        else:
+            dict_list = self.select_dict_list(sql=sql, params=params)
+            values = entity_list(dict_list=dict_list, cls=cls)
+            return values
+
+    def select_entity_first(self, cls=None, sql=None, params=None, like_entity=None, entity=None):
+        ll = self.select_entity_list(cls, sql=sql, params=params, like_entity=like_entity, entity=entity)
+        if ll is not None and len(ll) > 0:
+            return ll[0]
+
+    def select_entity_by_pk(self, entity, pk_value=None):
+        primary_key, _pk_value = get_entity_primary_key(entity)
+        table_name = getattr(entity, "__table_name__")
+        _SQL = ""
+        if isinstance(primary_key, list):
+            for pk in primary_key:
+                _SQL += f"""{pk}=:{pk} AND"""
+            _SQL = _SQL[:-3]
+        else:
+            _SQL = f"""{primary_key}=:{primary_key}"""
+        sql = f"""SELECT * FROM {table_name} WHERE {_SQL}"""
+        if pk_value is None:
+            pk_value = _pk_value
+
+        params = {}
+        if isinstance(pk_value, str):
+            params[primary_key] = pk_value
+        else:
+            params = pk_value
+
+        ll = self.select_entity_list(entity, sql=sql, params=params)
+        if ll is not None and len(ll) > 0:
+            return ll[0]
